@@ -4,47 +4,44 @@
 
     Builder superclass for all builders.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import pickle
 import time
 from os import path
-from typing import Any, Dict, Iterable, List, Sequence, Set, Tuple, Union
+from typing import (TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Sequence, Set, Tuple,
+                    Type, Union)
 
 from docutils import nodes
 from docutils.nodes import Node
 
 from sphinx.config import Config
-from sphinx.environment import BuildEnvironment, CONFIG_OK, CONFIG_CHANGED_REASON
+from sphinx.environment import CONFIG_CHANGED_REASON, CONFIG_OK, BuildEnvironment
 from sphinx.environment.adapters.asset import ImageAdapter
 from sphinx.errors import SphinxError
 from sphinx.events import EventManager
 from sphinx.io import read_doc
 from sphinx.locale import __
-from sphinx.util import import_object, logging, rst, progress_message, status_iterator
+from sphinx.util import import_object, logging, progress_message, rst, status_iterator
 from sphinx.util.build_phase import BuildPhase
 from sphinx.util.console import bold  # type: ignore
 from sphinx.util.docutils import sphinx_domains
 from sphinx.util.i18n import CatalogInfo, CatalogRepository, docname_to_domain
 from sphinx.util.osutil import SEP, ensuredir, relative_uri, relpath
-from sphinx.util.parallel import ParallelTasks, SerialTasks, make_chunks, \
-    parallel_available
+from sphinx.util.parallel import ParallelTasks, SerialTasks, make_chunks, parallel_available
 from sphinx.util.tags import Tags
 
 # side effect: registers roles and directives
-from sphinx import roles       # noqa
-from sphinx import directives  # noqa
-
+from sphinx import directives  # NOQA isort:skip
+from sphinx import roles  # NOQA isort:skip
 try:
     import multiprocessing
 except ImportError:
     multiprocessing = None
 
-if False:
-    # For type annotation
-    from typing import Type  # for python3.5.1
+if TYPE_CHECKING:
     from sphinx.application import Sphinx
 
 
@@ -67,7 +64,7 @@ class Builder:
 
     #: default translator class for the builder.  This can be overridden by
     #: :py:meth:`app.set_translator()`.
-    default_translator_class = None  # type: Type[nodes.NodeVisitor]
+    default_translator_class: Type[nodes.NodeVisitor] = None
     # doctree versioning method
     versioning_method = 'none'
     versioning_compare = False
@@ -78,7 +75,7 @@ class Builder:
 
     #: The list of MIME types of image formats supported by the builder.
     #: Image files are searched in the order in which they appear here.
-    supported_image_types = []  # type: List[str]
+    supported_image_types: List[str] = []
     #: The builder supports remote images or not.
     supported_remote_images = False
     #: The builder supports data URIs or not.
@@ -91,18 +88,18 @@ class Builder:
         self.doctreedir = app.doctreedir
         ensuredir(self.doctreedir)
 
-        self.app = app              # type: Sphinx
-        self.env = None             # type: BuildEnvironment
-        self.events = app.events    # type: EventManager
-        self.config = app.config    # type: Config
-        self.tags = app.tags        # type: Tags
+        self.app: Sphinx = app
+        self.env: Optional[BuildEnvironment] = None
+        self.events: EventManager = app.events
+        self.config: Config = app.config
+        self.tags: Tags = app.tags
         self.tags.add(self.format)
         self.tags.add(self.name)
         self.tags.add("format_%s" % self.format)
         self.tags.add("builder_%s" % self.name)
 
         # images that need to be copied over (source -> dest)
-        self.images = {}  # type: Dict[str, str]
+        self.images: Dict[str, str] = {}
         # basename of images directory
         self.imagedir = ""
         # relative path to image directory from current docname (used at writing docs)
@@ -110,7 +107,7 @@ class Builder:
 
         # these get set later
         self.parallel_ok = False
-        self.finish_tasks = None  # type: Any
+        self.finish_tasks: Any = None
 
     def set_environment(self, env: BuildEnvironment) -> None:
         """Store BuildEnvironment object."""
@@ -118,11 +115,11 @@ class Builder:
         self.env.set_versioning_method(self.versioning_method,
                                        self.versioning_compare)
 
-    def get_translator_class(self, *args) -> "Type[nodes.NodeVisitor]":
+    def get_translator_class(self, *args: Any) -> Type[nodes.NodeVisitor]:
         """Return a class of translator."""
         return self.app.registry.get_translator_class(self)
 
-    def create_translator(self, *args) -> nodes.NodeVisitor:
+    def create_translator(self, *args: Any) -> nodes.NodeVisitor:
         """Return an instance of translator.
 
         This method returns an instance of ``default_translator_class`` by default.
@@ -229,7 +226,7 @@ class Builder:
         self.compile_catalogs(set(repo.catalogs), message)
 
     def compile_specific_catalogs(self, specified_files: List[str]) -> None:
-        def to_domain(fpath: str) -> str:
+        def to_domain(fpath: str) -> Optional[str]:
             docname = self.env.path2doc(path.abspath(fpath))
             if docname:
                 return docname_to_domain(docname, self.config.gettext_compact)
@@ -265,8 +262,7 @@ class Builder:
         # relative to the source directory and without source_suffix.
         dirlen = len(self.srcdir) + 1
         to_write = []
-        suffixes = None  # type: Tuple[str]
-        suffixes = tuple(self.config.source_suffix)  # type: ignore
+        suffixes: Tuple[str] = tuple(self.config.source_suffix)  # type: ignore
         for filename in filenames:
             filename = path.normpath(path.abspath(filename))
             if not filename.startswith(self.srcdir):
@@ -379,7 +375,7 @@ class Builder:
         added, changed, removed = self.env.get_outdated_files(updated)
 
         # allow user intervention as well
-        for docs in self.events.emit('env-get-outdated', self, added, changed, removed):
+        for docs in self.events.emit('env-get-outdated', self.env, added, changed, removed):
             changed.update(set(docs) & self.env.found_docs)
 
         # if files were added or removed, all documents with globbed toctrees
@@ -417,9 +413,9 @@ class Builder:
         else:
             self._read_serial(docnames)
 
-        if self.config.master_doc not in self.env.all_docs:
-            raise SphinxError('master file %s not found' %
-                              self.env.doc2path(self.config.master_doc))
+        if self.config.root_doc not in self.env.all_docs:
+            raise SphinxError('root file %s not found' %
+                              self.env.doc2path(self.config.root_doc))
 
         for retval in self.events.emit('env-updated', self.env):
             if retval is not None:
@@ -521,7 +517,7 @@ class Builder:
             for tocdocname in self.env.files_to_rebuild.get(docname, set()):
                 if tocdocname in self.env.found_docs:
                     docnames.add(tocdocname)
-        docnames.add(self.config.master_doc)
+        docnames.add(self.config.root_doc)
 
         with progress_message(__('preparing documents')):
             self.prepare_writing(docnames)
