@@ -4,30 +4,23 @@
 
     Create a full-text search index for offline search.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import html
 import pickle
 import re
-import warnings
 from importlib import import_module
 from os import path
+from typing import IO, Any, Dict, Iterable, List, Optional, Set, Tuple, Type
 
 from docutils import nodes
+from docutils.nodes import Node
 
-from sphinx import addnodes
-from sphinx import package_dir
-from sphinx.deprecation import RemovedInSphinx40Warning
+from sphinx import addnodes, package_dir
+from sphinx.environment import BuildEnvironment
 from sphinx.search.jssplitter import splitter_code
-from sphinx.util import jsdump, rpartition
-
-if False:
-    # For type annotation
-    from typing import Any, Dict, IO, Iterable, List, Tuple, Set  # NOQA
-    from typing import Type  # for python3.5.1
-    from docutils import nodes  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
+from sphinx.util import jsdump
 
 
 class SearchLanguage:
@@ -43,6 +36,14 @@ class SearchLanguage:
        This is a set of stop words of the target language.  Default `stopwords`
        is empty.  This word is used for building index and embedded in JS.
 
+    .. attribute:: js_splitter_code
+
+       Return splitter function of JavaScript version.  The function should be
+       named as ``splitQuery``.  And it should take a string and return list of
+       strings.
+
+       .. versionadded:: 3.0
+
     .. attribute:: js_stemmer_code
 
        Return stemmer class of JavaScript version.  This class' name should be
@@ -52,10 +53,11 @@ class SearchLanguage:
        This class is used to preprocess search word which Sphinx HTML readers
        type, before searching index. Default implementation does nothing.
     """
-    lang = None                 # type: str
-    language_name = None        # type: str
-    stopwords = set()           # type: Set[str]
-    js_stemmer_rawcode = None   # type: str
+    lang: str = None
+    language_name: str = None
+    stopwords: Set[str] = set()
+    js_splitter_code: str = None
+    js_stemmer_rawcode: str = None
     js_stemmer_code = """
 /**
  * Dummy stemmer for languages without stemming rules.
@@ -69,19 +71,16 @@ var Stemmer = function() {
 
     _word_re = re.compile(r'(?u)\w+')
 
-    def __init__(self, options):
-        # type: (Dict) -> None
+    def __init__(self, options: Dict) -> None:
         self.options = options
         self.init(options)
 
-    def init(self, options):
-        # type: (Dict) -> None
+    def init(self, options: Dict) -> None:
         """
         Initialize the class with the options the user has given.
         """
 
-    def split(self, input):
-        # type: (str) -> List[str]
+    def split(self, input: str) -> List[str]:
         """
         This method splits a sentence into words.  Default splitter splits input
         at white spaces, which should be enough for most languages except CJK
@@ -89,8 +88,7 @@ var Stemmer = function() {
         """
         return self._word_re.findall(input)
 
-    def stem(self, word):
-        # type: (str) -> str
+    def stem(self, word: str) -> str:
         """
         This method implements stemming algorithm of the Python version.
 
@@ -103,8 +101,7 @@ var Stemmer = function() {
         """
         return word
 
-    def word_filter(self, word):
-        # type: (str) -> bool
+    def word_filter(self, word: str) -> bool:
         """
         Return true if the target word should be registered in the search index.
         This method is called after stemming.
@@ -113,7 +110,7 @@ var Stemmer = function() {
             len(word) == 0 or not (
                 ((len(word) < 3) and (12353 < ord(word[0]) < 12436)) or
                 (ord(word[0]) < 256 and (
-                    len(word) < 3 or word in self.stopwords or word.isdigit()
+                    word in self.stopwords
                 ))))
 
 
@@ -121,14 +118,13 @@ var Stemmer = function() {
 from sphinx.search.en import SearchEnglish
 
 
-def parse_stop_word(source):
-    # type: (str) -> Set[str]
+def parse_stop_word(source: str) -> Set[str]:
     """
-    parse snowball style word list like this:
+    Parse snowball style word list like this:
 
     * http://snowball.tartarus.org/algorithms/finnish/stop.txt
     """
-    result = set()  # type: Set[str]
+    result: Set[str] = set()
     for line in source.splitlines():
         line = line.split('|')[0]  # remove comment
         result.update(line.split())
@@ -136,7 +132,7 @@ def parse_stop_word(source):
 
 
 # maps language name to module.class or directly a class
-languages = {
+languages: Dict[str, Any] = {
     'da': 'sphinx.search.da.SearchDanish',
     'de': 'sphinx.search.de.SearchGerman',
     'en': SearchEnglish,
@@ -154,36 +150,32 @@ languages = {
     'sv': 'sphinx.search.sv.SearchSwedish',
     'tr': 'sphinx.search.tr.SearchTurkish',
     'zh': 'sphinx.search.zh.SearchChinese',
-}  # type: Dict[str, Any]
+}
 
 
 class _JavaScriptIndex:
     """
-    The search index as javascript file that calls a function
+    The search index as JavaScript file that calls a function
     on the documentation search object to register the index.
     """
 
     PREFIX = 'Search.setIndex('
     SUFFIX = ')'
 
-    def dumps(self, data):
-        # type: (Any) -> str
+    def dumps(self, data: Any) -> str:
         return self.PREFIX + jsdump.dumps(data) + self.SUFFIX
 
-    def loads(self, s):
-        # type: (str) -> Any
+    def loads(self, s: str) -> Any:
         data = s[len(self.PREFIX):-len(self.SUFFIX)]
         if not data or not s.startswith(self.PREFIX) or not \
            s.endswith(self.SUFFIX):
             raise ValueError('invalid data')
         return jsdump.loads(data)
 
-    def dump(self, data, f):
-        # type: (Any, IO) -> None
+    def dump(self, data: Any, f: IO) -> None:
         f.write(self.dumps(data))
 
-    def load(self, f):
-        # type: (IO) -> Any
+    def load(self, f: IO) -> Any:
         return self.loads(f.read())
 
 
@@ -195,19 +187,13 @@ class WordCollector(nodes.NodeVisitor):
     A special visitor that collects words for the `IndexBuilder`.
     """
 
-    def __init__(self, document, lang):
-        # type: (nodes.document, SearchLanguage) -> None
+    def __init__(self, document: nodes.document, lang: SearchLanguage) -> None:
         super().__init__(document)
-        self.found_words = []           # type: List[str]
-        self.found_title_words = []     # type: List[str]
+        self.found_words: List[str] = []
+        self.found_title_words: List[str] = []
         self.lang = lang
 
-    def is_meta_keywords(self, node, nodetype=None):
-        # type: (addnodes.meta, Any) -> bool
-        if nodetype is not None:
-            warnings.warn('"nodetype" argument for WordCollector.is_meta_keywords() '
-                          'is deprecated.', RemovedInSphinx40Warning)
-
+    def is_meta_keywords(self, node: addnodes.meta) -> bool:
         if isinstance(node, addnodes.meta) and node.get('name') == 'keywords':
             meta_lang = node.get('lang')
             if meta_lang is None:  # lang not specified
@@ -217,8 +203,7 @@ class WordCollector(nodes.NodeVisitor):
 
         return False
 
-    def dispatch_visit(self, node):
-        # type: (nodes.Node) -> None
+    def dispatch_visit(self, node: Node) -> None:
         if isinstance(node, nodes.comment):
             raise nodes.SkipNode
         elif isinstance(node, nodes.raw):
@@ -243,7 +228,7 @@ class WordCollector(nodes.NodeVisitor):
 
 class IndexBuilder:
     """
-    Helper class that creates a searchindex based on the doctrees
+    Helper class that creates a search index based on the doctrees
     passed to the `feed` method.
     """
     formats = {
@@ -251,32 +236,26 @@ class IndexBuilder:
         'pickle':   pickle
     }
 
-    def __init__(self, env, lang, options, scoring):
-        # type: (BuildEnvironment, str, Dict, str) -> None
+    def __init__(self, env: BuildEnvironment, lang: str, options: Dict, scoring: str) -> None:
         self.env = env
-        self._titles = {}           # type: Dict[str, str]
-                                    # docname -> title
-        self._filenames = {}        # type: Dict[str, str]
-                                    # docname -> filename
-        self._mapping = {}          # type: Dict[str, Set[str]]
-                                    # stemmed word -> set(docname)
-        self._title_mapping = {}    # type: Dict[str, Set[str]]
-                                    # stemmed words in titles -> set(docname)
-        self._stem_cache = {}       # type: Dict[str, str]
-                                    # word -> stemmed word
-        self._objtypes = {}         # type: Dict[Tuple[str, str], int]
-                                    # objtype -> index
-        self._objnames = {}         # type: Dict[int, Tuple[str, str, str]]
-                                    # objtype index -> (domain, type, objname (localized))
-        lang_class = languages.get(lang)    # type: Type[SearchLanguage]
-                                            # add language-specific SearchLanguage instance
+        self._titles: Dict[str, str] = {}           # docname -> title
+        self._filenames: Dict[str, str] = {}        # docname -> filename
+        self._mapping: Dict[str, Set[str]] = {}     # stemmed word -> set(docname)
+        # stemmed words in titles -> set(docname)
+        self._title_mapping: Dict[str, Set[str]] = {}
+        self._stem_cache: Dict[str, str] = {}       # word -> stemmed word
+        self._objtypes: Dict[Tuple[str, str], int] = {}     # objtype -> index
+        # objtype index -> (domain, type, objname (localized))
+        self._objnames: Dict[int, Tuple[str, str, str]] = {}
+        # add language-specific SearchLanguage instance
+        lang_class: Type[SearchLanguage] = languages.get(lang)
 
         # fallback; try again with language-code
         if lang_class is None and '_' in lang:
             lang_class = languages.get(lang.split('_')[0])
 
         if lang_class is None:
-            self.lang = SearchEnglish(options)  # type: SearchLanguage
+            self.lang: SearchLanguage = SearchEnglish(options)
         elif isinstance(lang_class, str):
             module, classname = lang_class.rsplit('.', 1)
             lang_class = getattr(import_module(module), classname)
@@ -292,8 +271,7 @@ class IndexBuilder:
             self.js_scorer_code = ''
         self.js_splitter_code = splitter_code
 
-    def load(self, stream, format):
-        # type: (IO, Any) -> None
+    def load(self, stream: IO, format: Any) -> None:
         """Reconstruct from frozen data."""
         if isinstance(format, str):
             format = self.formats[format]
@@ -306,8 +284,7 @@ class IndexBuilder:
         self._filenames = dict(zip(index2fn, frozen['filenames']))
         self._titles = dict(zip(index2fn, frozen['titles']))
 
-        def load_terms(mapping):
-            # type: (Dict[str, Any]) -> Dict[str, Set[str]]
+        def load_terms(mapping: Dict[str, Any]) -> Dict[str, Set[str]]:
             rv = {}
             for k, v in mapping.items():
                 if isinstance(v, int):
@@ -320,16 +297,15 @@ class IndexBuilder:
         self._title_mapping = load_terms(frozen['titleterms'])
         # no need to load keywords/objtypes
 
-    def dump(self, stream, format):
-        # type: (IO, Any) -> None
+    def dump(self, stream: IO, format: Any) -> None:
         """Dump the frozen index to a stream."""
         if isinstance(format, str):
             format = self.formats[format]
         format.dump(self.freeze(), stream)
 
-    def get_objects(self, fn2index):
-        # type: (Dict[str, int]) -> Dict[str, Dict[str, Tuple[int, int, int, str]]]
-        rv = {}  # type: Dict[str, Dict[str, Tuple[int, int, int, str]]]
+    def get_objects(self, fn2index: Dict[str, int]
+                    ) -> Dict[str, Dict[str, Tuple[int, int, int, str]]]:
+        rv: Dict[str, Dict[str, Tuple[int, int, int, str]]] = {}
         otypes = self._objtypes
         onames = self._objnames
         for domainname, domain in sorted(self.env.domains.items()):
@@ -341,7 +317,7 @@ class IndexBuilder:
                     continue
                 fullname = html.escape(fullname)
                 dispname = html.escape(dispname)
-                prefix, name = rpartition(dispname, '.')
+                prefix, _, name = dispname.rpartition('.')
                 pdict = rv.setdefault(prefix, {})
                 try:
                     typeindex = otypes[domainname, type]
@@ -364,9 +340,8 @@ class IndexBuilder:
                 pdict[name] = (fn2index[docname], typeindex, prio, shortanchor)
         return rv
 
-    def get_terms(self, fn2index):
-        # type: (Dict) -> Tuple[Dict[str, List[str]], Dict[str, List[str]]]
-        rvs = {}, {}  # type: Tuple[Dict[str, List[str]], Dict[str, List[str]]]
+    def get_terms(self, fn2index: Dict) -> Tuple[Dict[str, List[str]], Dict[str, List[str]]]:
+        rvs: Tuple[Dict[str, List[str]], Dict[str, List[str]]] = ({}, {})
         for rv, mapping in zip(rvs, (self._mapping, self._title_mapping)):
             for k, v in mapping.items():
                 if len(v) == 1:
@@ -377,8 +352,7 @@ class IndexBuilder:
                     rv[k] = sorted([fn2index[fn] for fn in v if fn in fn2index])
         return rvs
 
-    def freeze(self):
-        # type: () -> Dict[str, Any]
+    def freeze(self) -> Dict[str, Any]:
         """Create a usable data structure for serializing."""
         docnames, titles = zip(*sorted(self._titles.items()))
         filenames = [self._filenames.get(docname) for docname in docnames]
@@ -392,12 +366,10 @@ class IndexBuilder:
                     objects=objects, objtypes=objtypes, objnames=objnames,
                     titleterms=title_terms, envversion=self.env.version)
 
-    def label(self):
-        # type: () -> str
+    def label(self) -> str:
         return "%s (code: %s)" % (self.lang.language_name, self.lang.lang)
 
-    def prune(self, docnames):
-        # type: (Iterable[str]) -> None
+    def prune(self, docnames: Iterable[str]) -> None:
         """Remove data for all docnames not in the list."""
         new_titles = {}
         new_filenames = {}
@@ -412,8 +384,7 @@ class IndexBuilder:
         for wordnames in self._title_mapping.values():
             wordnames.intersection_update(docnames)
 
-    def feed(self, docname, filename, title, doctree):
-        # type: (str, str, str, nodes.document) -> None
+    def feed(self, docname: str, filename: str, title: str, doctree: nodes.document) -> None:
         """Feed a doctree to the index."""
         self._titles[docname] = title
         self._filenames[docname] = filename
@@ -422,8 +393,7 @@ class IndexBuilder:
         doctree.walk(visitor)
 
         # memoize self.lang.stem
-        def stem(word):
-            # type: (str) -> str
+        def stem(word: str) -> str:
             try:
                 return self._stem_cache[word]
             except KeyError:
@@ -447,19 +417,41 @@ class IndexBuilder:
             if _filter(stemmed_word) and not already_indexed:
                 self._mapping.setdefault(stemmed_word, set()).add(docname)
 
-    def context_for_searchtool(self):
-        # type: () -> Dict[str, Any]
+    def context_for_searchtool(self) -> Dict[str, Any]:
+        if self.lang.js_splitter_code:
+            js_splitter_code = self.lang.js_splitter_code
+        else:
+            js_splitter_code = self.js_splitter_code
+
         return {
-            'search_language_stemming_code': self.lang.js_stemmer_code,
+            'search_language_stemming_code': self.get_js_stemmer_code(),
             'search_language_stop_words': jsdump.dumps(sorted(self.lang.stopwords)),
             'search_scorer_tool': self.js_scorer_code,
-            'search_word_splitter_code': self.js_splitter_code,
+            'search_word_splitter_code': js_splitter_code,
         }
 
-    def get_js_stemmer_rawcode(self):
-        # type: () -> str
+    def get_js_stemmer_rawcodes(self) -> List[str]:
+        """Returns a list of non-minified stemmer JS files to copy."""
         if self.lang.js_stemmer_rawcode:
-            return path.join(package_dir, 'search', 'non-minified-js',
-                             self.lang.js_stemmer_rawcode)
+            return [
+                path.join(package_dir, 'search', 'non-minified-js', fname)
+                for fname in ('base-stemmer.js', self.lang.js_stemmer_rawcode)
+            ]
         else:
-            return None
+            return []
+
+    def get_js_stemmer_rawcode(self) -> Optional[str]:
+        return None
+
+    def get_js_stemmer_code(self) -> str:
+        """Returns JS code that will be inserted into language_data.js."""
+        if self.lang.js_stemmer_rawcode:
+            js_dir = path.join(package_dir, 'search', 'minified-js')
+            with open(path.join(js_dir, 'base-stemmer.js')) as js_file:
+                base_js = js_file.read()
+            with open(path.join(js_dir, self.lang.js_stemmer_rawcode)) as js_file:
+                language_js = js_file.read()
+            return ('%s\n%s\nStemmer = %sStemmer;' %
+                    (base_js, language_js, self.lang.language_name))
+        else:
+            return self.lang.js_stemmer_code
