@@ -4,14 +4,14 @@
 
     Docutils node-related utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
-import warnings
-from typing import Any, Callable, Iterable, List, Set, Tuple
-from typing import cast
+import unicodedata
+from typing import (TYPE_CHECKING, Any, Callable, Iterable, List, Optional, Set, Tuple, Type,
+                    Union, cast)
 
 from docutils import nodes
 from docutils.nodes import Element, Node
@@ -20,15 +20,14 @@ from docutils.parsers.rst.states import Inliner
 from docutils.statemachine import StringList
 
 from sphinx import addnodes
-from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.locale import __
 from sphinx.util import logging
 
-if False:
-    # For type annotation
-    from typing import Type  # for python3.5.1
+if TYPE_CHECKING:
     from sphinx.builders import Builder
-    from sphinx.utils.tags import Tags
+    from sphinx.domain import IndexEntry
+    from sphinx.environment import BuildEnvironment
+    from sphinx.util.tags import Tags
 
 logger = logging.getLogger(__name__)
 
@@ -41,8 +40,8 @@ caption_ref_re = explicit_title_re  # b/w compat alias
 class NodeMatcher:
     """A helper class for Node.traverse().
 
-    It checks that given node is an instance of specified node-classes and it has
-    specified node-attributes.
+    It checks that the given node is an instance of the specified node-classes and
+    has the specified node-attributes.
 
     For example, following example searches ``reference`` node having ``refdomain``
     and ``reftype`` attributes::
@@ -60,8 +59,8 @@ class NodeMatcher:
         # => [<reference ...>, <reference ...>, ...]
     """
 
-    def __init__(self, *classes: "Type[Node]", **attrs) -> None:
-        self.classes = classes
+    def __init__(self, *node_classes: Type[Node], **attrs: Any) -> None:
+        self.classes = node_classes
         self.attrs = attrs
 
     def match(self, node: Node) -> bool:
@@ -92,7 +91,7 @@ class NodeMatcher:
 
 def get_full_module_name(node: Node) -> str:
     """
-    return full module dotted path like: 'docutils.nodes.paragraph'
+    Return full module dotted path like: 'docutils.nodes.paragraph'
 
     :param nodes.Node node: target node
     :return: full module dotted path
@@ -172,7 +171,7 @@ def apply_source_workaround(node: Element) -> None:
     ))):
         logger.debug('[i18n] PATCH: %r to have source and line: %s',
                      get_full_module_name(node), repr_domxml(node))
-        node.source = get_node_source(node)
+        node.source = get_node_source(node) or ''
         node.line = 0  # need fix docutils to get `node.line`
         return
 
@@ -198,7 +197,11 @@ def is_translatable(node: Node) -> bool:
     if isinstance(node, addnodes.translatable):
         return True
 
-    if isinstance(node, nodes.Inline) and 'translatable' not in node:
+    # image node marked as translatable or having alt text
+    if isinstance(node, nodes.image) and (node.get('translatable') or node.get('alt')):
+        return True
+
+    if isinstance(node, nodes.Inline) and 'translatable' not in node:  # type: ignore
         # inline node must not be translated if 'translatable' is not set
         return False
 
@@ -225,9 +228,6 @@ def is_translatable(node: Node) -> bool:
             return False
         return True
 
-    if isinstance(node, nodes.image) and node.get('translatable'):
-        return True
-
     if isinstance(node, addnodes.meta):
         return True
     if is_pending_meta(node):
@@ -239,6 +239,7 @@ def is_translatable(node: Node) -> bool:
 LITERAL_TYPE_NODES = (
     nodes.literal_block,
     nodes.doctest_block,
+    nodes.math_block,
     nodes.raw,
 )
 IMAGE_TYPE_NODES = (
@@ -251,7 +252,7 @@ META_TYPE_NODES = (
 
 def extract_messages(doctree: Element) -> Iterable[Tuple[Element, str]]:
     """Extract translatable messages from a document tree."""
-    for node in doctree.traverse(is_translatable):  # type: nodes.Element
+    for node in doctree.traverse(is_translatable):  # type: Element
         if isinstance(node, addnodes.translatable):
             for msg in node.extract_original_messages():
                 yield node, msg
@@ -260,10 +261,13 @@ def extract_messages(doctree: Element) -> Iterable[Tuple[Element, str]]:
             msg = node.rawsource
             if not msg:
                 msg = node.astext()
-        elif isinstance(node, IMAGE_TYPE_NODES):
-            msg = '.. image:: %s' % node['uri']
+        elif isinstance(node, nodes.image):
             if node.get('alt'):
-                msg += '\n   :alt: %s' % node['alt']
+                yield node, node['alt']
+            if node.get('translatable'):
+                msg = '.. image:: %s' % node['uri']
+            else:
+                msg = ''
         elif isinstance(node, META_TYPE_NODES):
             msg = node.rawcontent
         elif isinstance(node, nodes.pending) and is_pending_meta(node):
@@ -276,20 +280,14 @@ def extract_messages(doctree: Element) -> Iterable[Tuple[Element, str]]:
             yield node, msg
 
 
-def find_source_node(node: Element) -> str:
-    warnings.warn('find_source_node() is deprecated.',
-                  RemovedInSphinx40Warning)
-    return get_node_source(node)
-
-
-def get_node_source(node: Element) -> str:
+def get_node_source(node: Element) -> Optional[str]:
     for pnode in traverse_parent(node):
         if pnode.source:
             return pnode.source
     return None
 
 
-def get_node_line(node: Element) -> int:
+def get_node_line(node: Element) -> Optional[int]:
     for pnode in traverse_parent(node):
         if pnode.line:
             return pnode.line
@@ -303,7 +301,7 @@ def traverse_parent(node: Element, cls: Any = None) -> Iterable[Element]:
         node = node.parent
 
 
-def get_prev_node(node: Node) -> Node:
+def get_prev_node(node: Node) -> Optional[Node]:
     pos = node.parent.index(node)
     if pos > 0:
         return node.parent[pos - 1]
@@ -311,7 +309,7 @@ def get_prev_node(node: Node) -> Node:
         return None
 
 
-def traverse_translatable_index(doctree: Element) -> Iterable[Tuple[Element, List[str]]]:
+def traverse_translatable_index(doctree: Element) -> Iterable[Tuple[Element, List["IndexEntry"]]]:  # NOQA
     """Traverse translatable index node from a document tree."""
     for node in doctree.traverse(NodeMatcher(addnodes.index, inline=False)):  # type: addnodes.index  # NOQA
         if 'raw_entries' in node:
@@ -363,10 +361,11 @@ indextypes = [
 ]
 
 
-def process_index_entry(entry: str, targetid: str) -> List[Tuple[str, str, str, str, str]]:
+def process_index_entry(entry: str, targetid: str
+                        ) -> List[Tuple[str, str, str, str, Optional[str]]]:
     from sphinx.domains.python import pairindextypes
 
-    indexentries = []  # type: List[Tuple[str, str, str, str, str]]
+    indexentries: List[Tuple[str, str, str, str, Optional[str]]] = []
     entry = entry.strip()
     oentry = entry
     main = ''
@@ -435,8 +434,118 @@ def inline_all_toctrees(builder: "Builder", docnameset: Set[str], docname: str,
     return tree
 
 
+def _make_id(string: str) -> str:
+    """Convert `string` into an identifier and return it.
+
+    This function is a modified version of ``docutils.nodes.make_id()`` of
+    docutils-0.16.
+
+    Changes:
+
+    * Allow to use capital alphabet characters
+    * Allow to use dots (".") and underscores ("_") for an identifier
+      without a leading character.
+
+    # Author: David Goodger <goodger@python.org>
+    # Maintainer: docutils-develop@lists.sourceforge.net
+    # Copyright: This module has been placed in the public domain.
+    """
+    id = string.translate(_non_id_translate_digraphs)
+    id = id.translate(_non_id_translate)
+    # get rid of non-ascii characters.
+    # 'ascii' lowercase to prevent problems with turkish locale.
+    id = unicodedata.normalize('NFKD', id).encode('ascii', 'ignore').decode('ascii')
+    # shrink runs of whitespace and replace by hyphen
+    id = _non_id_chars.sub('-', ' '.join(id.split()))
+    id = _non_id_at_ends.sub('', id)
+    return str(id)
+
+
+_non_id_chars = re.compile('[^a-zA-Z0-9._]+')
+_non_id_at_ends = re.compile('^[-0-9._]+|-+$')
+_non_id_translate = {
+    0x00f8: 'o',       # o with stroke
+    0x0111: 'd',       # d with stroke
+    0x0127: 'h',       # h with stroke
+    0x0131: 'i',       # dotless i
+    0x0142: 'l',       # l with stroke
+    0x0167: 't',       # t with stroke
+    0x0180: 'b',       # b with stroke
+    0x0183: 'b',       # b with topbar
+    0x0188: 'c',       # c with hook
+    0x018c: 'd',       # d with topbar
+    0x0192: 'f',       # f with hook
+    0x0199: 'k',       # k with hook
+    0x019a: 'l',       # l with bar
+    0x019e: 'n',       # n with long right leg
+    0x01a5: 'p',       # p with hook
+    0x01ab: 't',       # t with palatal hook
+    0x01ad: 't',       # t with hook
+    0x01b4: 'y',       # y with hook
+    0x01b6: 'z',       # z with stroke
+    0x01e5: 'g',       # g with stroke
+    0x0225: 'z',       # z with hook
+    0x0234: 'l',       # l with curl
+    0x0235: 'n',       # n with curl
+    0x0236: 't',       # t with curl
+    0x0237: 'j',       # dotless j
+    0x023c: 'c',       # c with stroke
+    0x023f: 's',       # s with swash tail
+    0x0240: 'z',       # z with swash tail
+    0x0247: 'e',       # e with stroke
+    0x0249: 'j',       # j with stroke
+    0x024b: 'q',       # q with hook tail
+    0x024d: 'r',       # r with stroke
+    0x024f: 'y',       # y with stroke
+}
+_non_id_translate_digraphs = {
+    0x00df: 'sz',      # ligature sz
+    0x00e6: 'ae',      # ae
+    0x0153: 'oe',      # ligature oe
+    0x0238: 'db',      # db digraph
+    0x0239: 'qp',      # qp digraph
+}
+
+
+def make_id(env: "BuildEnvironment", document: nodes.document,
+            prefix: str = '', term: str = None) -> str:
+    """Generate an appropriate node_id for given *prefix* and *term*."""
+    node_id = None
+    if prefix:
+        idformat = prefix + "-%s"
+    else:
+        idformat = (document.settings.id_prefix or "id") + "%s"
+
+    # try to generate node_id by *term*
+    if prefix and term:
+        node_id = _make_id(idformat % term)
+        if node_id == prefix:
+            # *term* is not good to generate a node_id.
+            node_id = None
+    elif term:
+        node_id = _make_id(term)
+        if node_id == '':
+            node_id = None  # fallback to None
+
+    while node_id is None or node_id in document.ids:
+        node_id = idformat % env.new_serialno(prefix)
+
+    return node_id
+
+
+def find_pending_xref_condition(node: addnodes.pending_xref, condition: str
+                                ) -> Optional[Element]:
+    """Pick matched pending_xref_condition node up from the pending_xref."""
+    for subnode in node:
+        if (isinstance(subnode, addnodes.pending_xref_condition) and
+                subnode.get('condition') == condition):
+            return subnode
+    else:
+        return None
+
+
 def make_refnode(builder: "Builder", fromdocname: str, todocname: str, targetid: str,
-                 child: Node, title: str = None) -> nodes.reference:
+                 child: Union[Node, List[Node]], title: str = None) -> nodes.reference:
     """Shortcut to create a reference node."""
     node = nodes.reference('', '', internal=True)
     if fromdocname == todocname and targetid:
@@ -449,7 +558,7 @@ def make_refnode(builder: "Builder", fromdocname: str, todocname: str, targetid:
             node['refuri'] = builder.get_relative_uri(fromdocname, todocname)
     if title:
         node['reftitle'] = title
-    node.append(child)
+    node += child
     return node
 
 
@@ -479,19 +588,21 @@ NON_SMARTQUOTABLE_PARENT_NODES = (
 
 
 def is_smartquotable(node: Node) -> bool:
-    """Check the node is smart-quotable or not."""
-    if isinstance(node.parent, NON_SMARTQUOTABLE_PARENT_NODES):
+    """Check whether the node is smart-quotable or not."""
+    for pnode in traverse_parent(node.parent):
+        if isinstance(pnode, NON_SMARTQUOTABLE_PARENT_NODES):
+            return False
+        elif pnode.get('support_smartquotes', None) is False:
+            return False
+
+    if getattr(node, 'support_smartquotes', None) is False:
         return False
-    elif node.parent.get('support_smartquotes', None) is False:
-        return False
-    elif getattr(node, 'support_smartquotes', None) is False:
-        return False
-    else:
-        return True
+
+    return True
 
 
 def process_only_nodes(document: Node, tags: "Tags") -> None:
-    """Filter ``only`` nodes which does not match *tags*."""
+    """Filter ``only`` nodes which do not match *tags*."""
     for node in document.traverse(addnodes.only):
         try:
             ret = tags.eval_condition(node['expr'])
@@ -510,10 +621,12 @@ def process_only_nodes(document: Node, tags: "Tags") -> None:
                 node.replace_self(nodes.comment())
 
 
-# monkey-patch Element.copy to copy the rawsource and line
-# for docutils-0.14 or older versions.
-
 def _new_copy(self: Element) -> Element:
+    """monkey-patch Element.copy to copy the rawsource and line
+    for docutils-0.16 or older versions.
+
+    refs: https://sourceforge.net/p/docutils/patches/165/
+    """
     newnode = self.__class__(self.rawsource, **self.attributes)
     if isinstance(self, nodes.Element):
         newnode.source = self.source
