@@ -4,7 +4,7 @@
 
     Test the sphinx.apidoc module.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -121,15 +121,15 @@ def test_pep_0420_enabled_separate(make_app, apidoc):
 
     with open(outdir / 'a.b.c.rst') as f:
         rst = f.read()
-        assert ".. toctree::\n\n   a.b.c.d\n" in rst
+        assert ".. toctree::\n   :maxdepth: 4\n\n   a.b.c.d\n" in rst
 
     with open(outdir / 'a.b.e.rst') as f:
         rst = f.read()
-        assert ".. toctree::\n\n   a.b.e.f\n" in rst
+        assert ".. toctree::\n   :maxdepth: 4\n\n   a.b.e.f\n" in rst
 
     with open(outdir / 'a.b.x.rst') as f:
         rst = f.read()
-        assert ".. toctree::\n\n   a.b.x.y\n" in rst
+        assert ".. toctree::\n   :maxdepth: 4\n\n   a.b.x.y\n" in rst
 
     app = make_app('text', srcdir=outdir)
     app.build()
@@ -216,6 +216,8 @@ def test_trailing_underscore(make_app, apidoc):
 def test_excludes(apidoc):
     outdir = apidoc.outdir
     assert (outdir / 'conf.py').isfile()
+    assert (outdir / 'a.rst').isfile()
+    assert (outdir / 'a.b.rst').isfile()
     assert (outdir / 'a.b.c.rst').isfile()  # generated because not empty
     assert not (outdir / 'a.b.e.rst').isfile()  # skipped because of empty after excludes
     assert (outdir / 'a.b.x.rst').isfile()
@@ -231,6 +233,8 @@ def test_excludes_subpackage_should_be_skipped(apidoc):
     """Subpackage exclusion should work."""
     outdir = apidoc.outdir
     assert (outdir / 'conf.py').isfile()
+    assert (outdir / 'a.rst').isfile()
+    assert (outdir / 'a.b.rst').isfile()
     assert (outdir / 'a.b.c.rst').isfile()  # generated because not empty
     assert not (outdir / 'a.b.e.f.rst').isfile()  # skipped because 'b/e' subpackage is skipped
 
@@ -244,6 +248,8 @@ def test_excludes_module_should_be_skipped(apidoc):
     """Module exclusion should work."""
     outdir = apidoc.outdir
     assert (outdir / 'conf.py').isfile()
+    assert (outdir / 'a.rst').isfile()
+    assert (outdir / 'a.b.rst').isfile()
     assert (outdir / 'a.b.c.rst').isfile()  # generated because not empty
     assert not (outdir / 'a.b.e.f.rst').isfile()  # skipped because of empty after excludes
 
@@ -257,6 +263,8 @@ def test_excludes_module_should_not_be_skipped(apidoc):
     """Module should be included if no excludes are used."""
     outdir = apidoc.outdir
     assert (outdir / 'conf.py').isfile()
+    assert (outdir / 'a.rst').isfile()
+    assert (outdir / 'a.b.rst').isfile()
     assert (outdir / 'a.b.c.rst').isfile()  # generated because not empty
     assert (outdir / 'a.b.e.f.rst').isfile()  # skipped because of empty after excludes
 
@@ -275,7 +283,7 @@ def test_multibyte_parameters(make_app, apidoc):
     assert (outdir / 'conf.py').isfile()
     assert (outdir / 'index.rst').isfile()
 
-    conf_py = (outdir / 'conf.py').text()
+    conf_py = (outdir / 'conf.py').read_text()
     assert "project = 'プロジェクト名'" in conf_py
     assert "author = '著者名'" in conf_py
     assert "version = 'バージョン'" in conf_py
@@ -408,11 +416,13 @@ def test_private(tempdir):
     # without --private option
     apidoc_main(['-o', tempdir, tempdir])
     assert (tempdir / 'hello.rst').exists()
+    assert ':private-members:' not in (tempdir / 'hello.rst').read_text()
     assert not (tempdir / '_world.rst').exists()
 
     # with --private option
-    apidoc_main(['--private', '-o', tempdir, tempdir])
+    apidoc_main(['--private', '-f', '-o', tempdir, tempdir])
     assert (tempdir / 'hello.rst').exists()
+    assert ':private-members:' in (tempdir / 'hello.rst').read_text()
     assert (tempdir / '_world.rst').exists()
 
 
@@ -424,7 +434,7 @@ def test_toc_file(tempdir):
     apidoc_main(['-o', tempdir, tempdir])
     assert (outdir / 'modules.rst').exists()
 
-    content = (outdir / 'modules.rst').text()
+    content = (outdir / 'modules.rst').read_text()
     assert content == ("test_toc_file0\n"
                        "==============\n"
                        "\n"
@@ -440,7 +450,7 @@ def test_module_file(tempdir):
     apidoc_main(['-o', tempdir, tempdir])
     assert (outdir / 'example.rst').exists()
 
-    content = (outdir / 'example.rst').text()
+    content = (outdir / 'example.rst').read_text()
     assert content == ("example module\n"
                        "==============\n"
                        "\n"
@@ -456,7 +466,7 @@ def test_module_file_noheadings(tempdir):
     apidoc_main(['--no-headings', '-o', tempdir, tempdir])
     assert (outdir / 'example.rst').exists()
 
-    content = (outdir / 'example.rst').text()
+    content = (outdir / 'example.rst').read_text()
     assert content == (".. automodule:: example\n"
                        "   :members:\n"
                        "   :undoc-members:\n"
@@ -475,7 +485,7 @@ def test_package_file(tempdir):
     assert (outdir / 'testpkg.rst').exists()
     assert (outdir / 'testpkg.subpkg.rst').exists()
 
-    content = (outdir / 'testpkg.rst').text()
+    content = (outdir / 'testpkg.rst').read_text()
     assert content == ("testpkg package\n"
                        "===============\n"
                        "\n"
@@ -483,6 +493,7 @@ def test_package_file(tempdir):
                        "-----------\n"
                        "\n"
                        ".. toctree::\n"
+                       "   :maxdepth: 4\n"
                        "\n"
                        "   testpkg.subpkg\n"
                        "\n"
@@ -505,7 +516,6 @@ def test_package_file(tempdir):
                        "   :undoc-members:\n"
                        "   :show-inheritance:\n"
                        "\n"
-                       "\n"
                        "Module contents\n"
                        "---------------\n"
                        "\n"
@@ -514,7 +524,7 @@ def test_package_file(tempdir):
                        "   :undoc-members:\n"
                        "   :show-inheritance:\n")
 
-    content = (outdir / 'testpkg.subpkg.rst').text()
+    content = (outdir / 'testpkg.subpkg.rst').read_text()
     assert content == ("testpkg.subpkg package\n"
                        "======================\n"
                        "\n"
@@ -536,7 +546,7 @@ def test_package_file_separate(tempdir):
     assert (outdir / 'testpkg.rst').exists()
     assert (outdir / 'testpkg.example.rst').exists()
 
-    content = (outdir / 'testpkg.rst').text()
+    content = (outdir / 'testpkg.rst').read_text()
     assert content == ("testpkg package\n"
                        "===============\n"
                        "\n"
@@ -544,6 +554,7 @@ def test_package_file_separate(tempdir):
                        "----------\n"
                        "\n"
                        ".. toctree::\n"
+                       "   :maxdepth: 4\n"
                        "\n"
                        "   testpkg.example\n"
                        "\n"
@@ -555,7 +566,7 @@ def test_package_file_separate(tempdir):
                        "   :undoc-members:\n"
                        "   :show-inheritance:\n")
 
-    content = (outdir / 'testpkg.example.rst').text()
+    content = (outdir / 'testpkg.example.rst').read_text()
     assert content == ("testpkg.example module\n"
                        "======================\n"
                        "\n"
@@ -572,7 +583,7 @@ def test_package_file_module_first(tempdir):
     (outdir / 'testpkg' / 'example.py').write_text('')
     apidoc_main(['--module-first', '-o', tempdir, tempdir])
 
-    content = (outdir / 'testpkg.rst').text()
+    content = (outdir / 'testpkg.rst').read_text()
     assert content == ("testpkg package\n"
                        "===============\n"
                        "\n"
@@ -590,8 +601,7 @@ def test_package_file_module_first(tempdir):
                        ".. automodule:: testpkg.example\n"
                        "   :members:\n"
                        "   :undoc-members:\n"
-                       "   :show-inheritance:\n"
-                       "\n")
+                       "   :show-inheritance:\n")
 
 
 def test_package_file_without_submodules(tempdir):
@@ -601,7 +611,7 @@ def test_package_file_without_submodules(tempdir):
     apidoc_main(['-o', tempdir, tempdir / 'testpkg'])
     assert (outdir / 'testpkg.rst').exists()
 
-    content = (outdir / 'testpkg.rst').text()
+    content = (outdir / 'testpkg.rst').read_text()
     assert content == ("testpkg package\n"
                        "===============\n"
                        "\n"
@@ -621,10 +631,12 @@ def test_namespace_package_file(tempdir):
     apidoc_main(['--implicit-namespace', '-o', tempdir, tempdir / 'testpkg'])
     assert (outdir / 'testpkg.rst').exists()
 
-    content = (outdir / 'testpkg.rst').text()
+    content = (outdir / 'testpkg.rst').read_text()
     assert content == ("testpkg namespace\n"
                        "=================\n"
                        "\n"
+                       ".. py:module:: testpkg\n"
+                       "\n"
                        "Submodules\n"
                        "----------\n"
                        "\n"
@@ -634,5 +646,4 @@ def test_namespace_package_file(tempdir):
                        ".. automodule:: testpkg.example\n"
                        "   :members:\n"
                        "   :undoc-members:\n"
-                       "   :show-inheritance:\n"
-                       "\n")
+                       "   :show-inheritance:\n")
